/*
	
	PasswordView.cp
	
	Copyright 1998, Neil Miller. All Rights Reserved.

	This source is free; you can redistribute it and/or modify it under 
	the terms of the a modified GNU Library General Public License OR 
	Mozilla Public License.
	See "http://www.pobox.com/~ngm/be/index.html" for full details.

*/

#include "PasswordView.h"

#include <Beep.h>
#include <Looper.h>
#include <string.h>
#include <stdio.h>

char*	StrDup( const char *string);	// As strdup() but uses new instead of malloc().

PasswordView::PasswordView(BRect frame, const char *name, BRect textRect,
								  uint32 resizingMode, uint32 flags) :
			BTextView(frame, name, textRect, resizingMode, flags)
{
	SetWordWrap( false );
	password = StrDup( "" );
	SetBlotChar( "*" );
	dragging = NULL;
}

PasswordView::PasswordView(BRect frame, const char *name, BRect textRect, 
						const BFont *font, const rgb_color *color, 
						uint32 resizingMode, uint32 flags) :
			BTextView(frame, name, textRect, font, color, 
						resizingMode, flags)
{
	SetWordWrap( false );
	password = StrDup( "" );
	SetBlotChar( "*" );
	dragging = NULL;
}

PasswordView::PasswordView(BMessage *data) : BTextView( data )
{
	SetWordWrap( false );
	password = StrDup( "" );
	SetBlotChar( "*" );
	dragging = NULL;
}

PasswordView::~PasswordView()
{
	int32	i = 0;
	
	if (dragging != NULL) {
		delete dragging;
	}
	
	while ( password[i] != '\0' ) {
		password[i] = 0;
		i++;
	}
	delete [] password;
	delete [] blot;
}	

const char* PasswordView::Password() const
{
	return password;
}

void PasswordView::SetBlotChar( const char *blotChar )
{
	if ( blot != NULL ) {
		delete [] blot;
	}
	blotBytes = strlen( blotChar );
	blot = StrDup( blotChar );
}

const char* PasswordView::GetBlotChar() const
{
	return blot;
}
		
void PasswordView::RequireASCII()
{
	// currently un-implemented, and is default action
}

void PasswordView::InsertText(const char *inText, int32 inLength, int32 inOffset,
								const text_run_array *inRuns)
{
	// WARNING ASSUMPTION: inText must be ASCII or it will not be accepted!
	BMessage	*msg;
	char		*temp;
	const char	*textHold;
	int32		 inLen;
	int32		 charOffset = inOffset / blotBytes; // get the character position of the offset
	int32		 passLen = strlen(password);
	
	if ( inLength > 1) {
		// might not be ASCII, check for charaters with high bit set (UTF-8)
		for ( inLen = 0; inLen < inLength; inLen++) {
			if ( inText[inLen] >= 128 )
				beep();
				// repostion insertion point to correct place.
				Select( inOffset - inLength, inOffset - inLength);
				return; // don't add text, it's UTF-8
		}
	}
	
	msg = Looper()->CurrentMessage();
	if ( msg != NULL ) {
		if ( msg->WasDropped() ) {
			textHold = inText;
			inLen = inLength;
			if ( dragging != NULL ) {
				inText = dragging;
				inLength = strlen( dragging );
			}
		}
	}
	
	// allocate new password
	temp = new char[ passLen + inLength + 1];
	// insert new text into password
	strncpy( temp, password, charOffset); // charOffset == index of char to insert before
	strncpy( temp + charOffset, inText, inLength );
	strncpy( temp + charOffset + inLength, password + charOffset, passLen - charOffset );
	// terminate new password
	temp[ passLen + inLength ] = '\0';
	
	// replace old password with new password.
	delete password;
	password = temp;
	if ( dragging != NULL ) {
		delete [] dragging;
		dragging = NULL;
	}
		
	if ( msg->WasDropped() && inText != textHold ) {
		BTextView::InsertText( textHold, inLen, inOffset, NULL);
	} else {
		// generate input for BTextView
		passLen = inLength; // reuse passLen for input len
		inLength = (inLength * blotBytes);
		temp = new char[ inLength + 1];
		for (int i = 0; i < inLength; i += blotBytes) {
			for (int j = 0; j < blotBytes; j++) {
				temp[i+j] = blot[j];
			}
		}
		temp[inLength] = '\0';
		
		BTextView::InsertText(temp, inLength, inOffset, NULL);
	
		Select( inOffset + inLength - 1, inOffset + inLength - 1);
		delete temp;
	}
}
								   
void PasswordView::DeleteText(int32 fromOffset, int32 toOffset)
{
	// WARNING ASSUMPTION: password must be ASCII !!
	BMessage	*msg;
	char		*temp;
	int32		 charOffset = fromOffset / blotBytes; // get the charater position of the offset
	int32		 deleteLen = (toOffset - fromOffset) / blotBytes;
	int32		 passLen = strlen(password);

	// check if the current message was dropped, if so this delete 
	// is to remove the dragged text from its current location
	msg = Looper()->CurrentMessage();
	if ( msg != NULL ) {
		if ( msg->WasDropped() ) {
			if ( dragging != NULL ) {
				delete dragging; // old drag, was not be deleted
			}
			dragging = new char[ deleteLen + 1];
			strncpy( dragging, password + charOffset, deleteLen );
			dragging[ deleteLen ] = '\0';
		}
	}	
	
	// allocate new password
	temp = new char[ passLen - deleteLen + 1];
	// remove deleted text from password
	strncpy( temp, password, charOffset); // charOffset == index of char to insert before
	strncpy( temp + charOffset, password + deleteLen, passLen - deleteLen );
	// terminate new password
	temp[ passLen - deleteLen ] = '\0';
	
	// replace old password with new password.
	delete password;
	password = temp;

	BTextView::DeleteText( fromOffset, toOffset );
}

void PasswordView::KeyDown(const char *bytes, int32 numBytes)
{
	if ( bytes[0] == B_TAB ) {
		// allow user to navigate via the keyborad.
		BView::KeyDown(bytes, numBytes);
	} else {
		BTextView::KeyDown(bytes, numBytes);
	}
}

char* StrDup( const char *string)
{
	if ( string == NULL ) {
		return NULL;
	}
	
	char *copy = new char[strlen(string) + 1];
	strcpy( copy, string);
	return copy;
}

// end